* CD0.F
* C0, C00, D0, D00 for zero momenta, from arXiv:0811.2891
* this file is part of LoopTools
* last modified 19 Feb 09 th


	double complex function C0Z(m1, m2, m3)
	implicit none
	double precision m1, m2, m3

#include "lt.h"

	double precision m(3), mx
	integer o

	double precision mlog, dq, ma, mb
	mlog(ma) = log(max(ma, eps))
	dq(ma, mb) = (ma*mlog(ma) - mb*mlog(mb))/(ma - mb)

	if( abs(m1 - m2) .lt. eps ) then
	  o = '231'O
	else if( abs(m1 - m3) .lt. eps ) then
	  o = '213'O
	else if( abs(m2 - m3) .lt. eps ) then
	  o = '123'O
	else
	  C0Z = (dq(m1, m3) - dq(m1, m2))/(m2 - m3)
	  return
	endif

	m(ibits(o, 6, 3)) = m1
	m(ibits(o, 3, 3)) = m2
	m(ibits(o, 0, 3)) = m3

	if( abs(m(1) - m(2)) .lt. eps ) then
	  C0Z = -.5D0/m(1)
	  return
	endif

	mx = 1/(m(1) - m(2))
	C0Z = mx*(1 - mx*m(1)*(mlog(m(1)) - mlog(m(2))))
	end

************************************************************************

	double complex function C00Z(m1, m2, m3)
	implicit none
	double precision m1, m2, m3

#include "lt.h"

	double precision m(3), mx
	integer o

	double precision mlog, dq, ma, mb
	mlog(ma) = log(max(ma, eps))
	dq(ma, mb) = (ma**2*mlog(ma) - mb**2*mlog(mb))/(ma - mb)

	if( abs(m1 - m2) .lt. eps ) then
	  o = '231'O
	else if( abs(m1 - m3) .lt. eps ) then
	  o = '213'O
	else if( abs(m2 - m3) .lt. eps ) then
	  o = '123'O
	else
	  C00Z = .25D0*(1 - (dq(m1, m3) - dq(m1, m2))/(m2 - m3))
	  return
	endif

	m(ibits(o, 6, 3)) = m1
	m(ibits(o, 3, 3)) = m2
	m(ibits(o, 0, 3)) = m3

	if( abs(m(1) - m(2)) .lt. eps ) then
	  C00Z = -.125D0 - .25D0*log(m(1))
	  return
	endif

	mx = m(1)/(m(1) - m(2))
	C00Z = .25D0*(mx*(1 - mx*mlog(m(1)/m(2))) - log(m(2)))
	end

************************************************************************

	double complex function D0Z(m1, m2, m3, m4)
	implicit none
	double precision m1, m2, m3, m4

#include "lt.h"

	double precision m(4), mx
	integer o

	double precision mlog, dq, dl, ma, mb, mc, md
	mlog(ma) = log(max(ma, eps))
	dq(ma, mb, mc, md) =
     &    -ma*mlog(ma)/((ma - mb)*(ma - mc)*(ma - md))
	dl(ma, mb, mc) =
     &    ((ma**2 - mb*mc)*mlog(ma) - (ma - mb)*(ma - mc))/
     &      ((ma - mb)*(ma - mc))**2

	if( abs(m1 - m2) .lt. eps ) then
	  o = '3412'O
	else if( abs(m1 - m3) .lt. eps ) then
	  o = '3142'O
	else if( abs(m1 - m4) .lt. eps ) then
	  o = '3124'O
	else if( abs(m2 - m3) .lt. eps ) then
	  o = '1342'O
	else if( abs(m2 - m4) .lt. eps ) then
	  o = '1324'O
	else if( abs(m3 - m4) .lt. eps ) then
	  o = '1234'O
	else
	  D0Z = dq(m1, m2, m3, m4) + dq(m2, m1, m3, m4) +
     &      dq(m3, m1, m2, m4) + dq(m4, m1, m2, m3)
	  return
	endif

	m(ibits(o, 9, 3)) = m1
	m(ibits(o, 6, 3)) = m2
	m(ibits(o, 3, 3)) = m3
	m(ibits(o, 0, 3)) = m4

	if( abs(m(1) - m(3)) .lt. eps ) then
	  mx = m(1)
	  m(1) = m(3)
	  m(3) = mx
	endif

	if( abs(m(1) - m(3)) .lt. eps ) then
	  D0Z = 1/6D0/m(1)**2
	  return
	endif

	if( abs(m(2) - m(3)) .lt. eps ) then
	  D0Z = (.5D0*(m(1) - m(2))*(m(1) + m(2))/m(2) -
     &      m(1)*mlog(m(1)/m(2)))/(m(1) - m(2))**3
	  return
	endif

	D0Z = dl(m(3), m(1), m(2))

	if( abs(m(1) - m(2)) .lt. eps ) then
	  D0Z = D0Z + dl(m(1), m(3), m(3))
	else
	  D0Z = D0Z + dq(m(1), m(2), m(3), m(3)) +
     &                dq(m(2), m(1), m(3), m(3))
	endif
	end

************************************************************************

	double complex function D00Z(m1, m2, m3, m4)
	implicit none
	double precision m1, m2, m3, m4

#include "lt.h"

	double precision m(4), mx
	integer o

	double precision mlog, dq, dl, ma, mb, mc, md
	mlog(ma) = log(max(ma, eps))
	dq(ma, mb, mc, md) =
     &    -.25D0*ma**2*mlog(ma)/((ma - mb)*(ma - mc)*(ma - md))
	dl(ma, mb, mc) = .25D0*ma*
     &    ((mc*(ma - mb) + mb*(ma - mc))*mlog(ma) -
     &       (ma - mb)*(ma - mc))/((ma - mb)*(ma - mc))**2

	if( abs(m1 - m2) .lt. eps ) then
	  o = '3412'O
	else if( abs(m1 - m3) .lt. eps ) then
	  o = '3142'O
	else if( abs(m1 - m4) .lt. eps ) then
	  o = '3124'O
	else if( abs(m2 - m3) .lt. eps ) then
	  o = '1342'O
	else if( abs(m2 - m4) .lt. eps ) then
	  o = '1324'O
	else if( abs(m3 - m4) .lt. eps ) then
	  o = '1234'O
	else
	  D00Z = dq(m1, m2, m3, m4) + dq(m2, m1, m3, m4) +
     &      dq(m3, m1, m2, m4) + dq(m4, m1, m2, m3)
	  return
	endif

	m(ibits(o, 9, 3)) = m1
	m(ibits(o, 6, 3)) = m2
	m(ibits(o, 3, 3)) = m3
	m(ibits(o, 0, 3)) = m4

	if( abs(m(1) - m(3)) .lt. eps ) then
	  mx = m(1)
	  m(1) = m(3)
	  m(3) = mx
	endif

	if( abs(m(1) - m(3)) .lt. eps ) then
	  D00Z = -1/12D0/m(1)
	  return
	endif

	if( abs(m(2) - m(3)) .lt. eps ) then
	  D00Z = .125*((2*m(1) - m(2))**2 -
     &      m(1)**2*(2*mlog(m(1)/m(2)) + 1))/(m(1) - m(2))**3
	  return
	endif

	D00Z = dl(m(3), m(1), m(2))

	if( abs(m(1) - m(2)) .lt. eps ) then
	  D00Z = D00Z + dl(m(1), m(3), m(3))
	else
	  D00Z = D00Z + dq(m(1), m(2), m(3), m(3)) +
     &                  dq(m(2), m(1), m(3), m(3))
	endif
	end

